//
//	DateTime.h
//
//	Автор	:	vagner
//	Создан	:	Jan 31, 2017
//

#pragma once

#include <string>
#include <time.h>

#include "vdef.h"
#include "vtime.h"

namespace vCommon
{
	//	Обертка секунд от начала.
	class DateTime
	{
	private:

		//	секунды от начала
		time_t usec;

		//	получить tm из секунд
		inline tm make_time ( ) const
		{
			return *gmtime ( &usec );
		}

		//	округлить
		inline void round ( time_t mod )
		{
	 		usec = usec - ( usec %  mod );
		}

	public:
		//	Формат времени : ЧЧ.ММ.СС
		static const std::string timeFormat;
		//	Формат даты : ДД.ММ.ГГГГ
		static const std::string dateFormat;
		//	Формат даты/времени : ДД.ММ.ГГГГ ЧЧ:ММ:СС
		static const std::string dtFormat;
		//	Формат даты/времени с милисекундами : ДД.ММ.ГГГГ ЧЧ:ММ:СС.мск
		static const std::string msFormat;

		inline int Min ( ) const
		{
			return make_time ( ).tm_min;
		}
		inline int Hour ( ) const
		{
			return make_time ( ).tm_hour;
		}
		inline int Day ( ) const
		{
			return make_time ( ).tm_mday;
		}
		inline int Month ( ) const
		{
			return make_time ( ).tm_mon + 1;
		}
		inline int WeekDay ( ) const
		{
			return make_time ( ).tm_wday;
		}
		inline int Sec ( ) const
		{
			return make_time ( ).tm_sec;
		}
		inline int Year ( ) const
		{
			return make_time ( ).tm_year + 1900;
		}
		inline int YearDay ( ) const
		{
			return make_time ( ).tm_yday + 1;
		}
		//	operators
		friend bool operator < ( const DateTime &l, const DateTime &r )
		{
			return l.usec < r.usec;
		}
		friend bool operator <= ( const DateTime &l, const DateTime &r )
		{
			return l.usec <= r.usec;
		}
		friend bool operator > ( const DateTime &l, const DateTime &r )
		{
			return l.usec > r.usec;
		}
		friend bool operator >= ( const DateTime &l, const DateTime &r )
		{
			return l.usec >= r.usec;
		}

		//	ctor
		DateTime ( );

		//	ctor
		DateTime ( time_t sec );

		//  текущее время
		static DateTime Now ( );

		//	добавить ко времени минуты
		void AddMin ( time_t min );

		//	дата/время в строковом представлении
		std::string Format ( const std::string &dtFormat ) const;

		//	округлить до дня
		DateTime DayRound ( );

		//	округлить до минут
		DateTime MinRound ( );

		time_t get_usec ( ) { return usec; }
	};

} /* namespace vCommon */
